#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import subprocess
from ikabot.helpers.signals import deactivate_sigint

def forkear(s):
	newpid = os.fork()
	if newpid != 0:
		# padre
		newpid = str(newpid)
		run('kill -SIGSTOP ' + newpid)
		run('bg ' + newpid)
		run('disown ' + newpid)
	else:
		# hijo
		s.padre = False
		deactivate_sigint()
		s.login()

def run(command):
	return subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE).stdout



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import gettext
import getpass
import ikabot.config as config
from ikabot.web.sesion import *
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.gui import banner

t = gettext.translation('getSesion', 
                        config.localedir, 
                        languages=config.idiomas,
                        fallback=True)
_ = t.gettext

def getSesion():
	banner()
	html = normal_get('https://es.ikariam.gameforge.com/?').text
	servidores = re.findall(r'<a href="(?:https:)?//(\w{2})\.ikariam\.gameforge\.com/\?kid=[\d\w-]*" target="_top" rel="nofollow" class="mmoflag mmo_\w{2}">(.+)</a>', html)
	i = 0
	for server in servidores:
		i += 1
		print('({:d}) {}'.format(i, server[1]))
	servidor = read(msg=_('Servidor:'), min=1, max=len(servidores))
	srv = servidores[servidor - 1][0]
	config.infoUser = _('Servidor:{}').format(servidores[servidor-1][1])
	banner()
	if srv != 'es':
		html = normal_get('https://{}.ikariam.gameforge.com/?'.format(srv)).text
	html = re.search(r'registerServer[\s\S]*registerServerServerInfo', html).group()
	mundos = re.findall(r'mobileUrl="s(\d{1,3})-\w{2}\.ikariam\.gameforge\.com"(?:\s*cookieName="")?\s*>\s*([\w\s]+?)\s*</option>', html)
	i = 0
	for mundo in mundos:
		i += 1
		print('({:d}) {}'.format(i, mundo[1]))
	mundo = read(msg=_('Mundo:'), min=1, max=len(mundos))
	mundo = mundos[mundo - 1]
	config.infoUser += _(', Mundo:{}').format(mundo[1])
	urlBase = 'http://s{}-{}.ikariam.gameforge.com/index.php?'.format(mundo[0], srv)
	if secure_traffic:
		urlBase.replace('http', 'https')
	uni_url = 's{}-{}.ikariam.gameforge.com'.format(mundo[0], srv)
	banner()
	usuario = read(msg=_('Usuario:'))
	password = getpass.getpass(_('Contraseña:'))
	headers = {'Host': uni_url, 'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:66.0) Gecko/20100101 Firefox/66.0','Accept':'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8', 'Accept-Encoding':'gzip, deflate, br','Referer': urlBase, 'DNT': '1', 'Connection': 'keep-alive', 'Upgrade-Insecure-Requests': '1'}
	payload = {'uni_url': uni_url, 'name': usuario, 'password': password, 'pwat_uid': '', 'pwat_checksum': '' ,'startPageShown' : '1' , 'detectedDevice' : '1' , 'kid':''}
	config.infoUser += _(', Jugador:{}').format(usuario)
	return Sesion(urlBase, payload, headers)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import gettext
from ikabot.config import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.getJson import *
from ikabot.helpers.recursos import *
from ikabot.helpers.gui import *
from ikabot.helpers.varios import addPuntos

t = gettext.translation('donar', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def donar(s):
	bienes = {'1': _('Viñedo'), '2': _('Cantera'), '3': _('Mina de cristal'), '4': _('Mina de azufre')}
	banner()

	ciudad = elegirCiudad(s)
	html = ciudad['html']
	banner()

	madera = getRecursosDisponibles(html)[0]
	almacenamiento = getCapacidadDeAlmacenamiento(html)

	idIsla = ciudad['islandId']
	html = s.get(urlIsla + idIsla)
	isla = getIsla(html)

	tipo = re.search(r'"tradegood":"(\d)"', html).group(1)
	bien = bienes[tipo]

	urlAserradero = 'view=resource&type=resource&islandId={0}&backgroundView=island&currentIslandId={0}&actionRequest={1}&ajax=1'.format(idIsla, s.token())
	aserraderoOk = printEstadoMina(s, urlAserradero, 'Aserradero')

	urlBien = 'view=tradegood&type={0}&islandId={1}&backgroundView=island&currentIslandId={1}&actionRequest={2}&ajax=1'.format(tipo, idIsla, s.token())
	bienOk = printEstadoMina(s, urlBien, bien)

	tipo = ['resource', 'tradegood']
	print(_('Madera disopnible:{} / {}\n').format(addPuntos(madera), addPuntos(almacenamiento)))

	if aserraderoOk is True and bienOk is True:
		msg = _('Aserradero(1) o {}(2)?:').format(bien)
		tipoDonacion = read(msg=msg, min=1, max=2)
	elif aserraderoOk is True and bienOk is False:
		tipoDonacion = 1
		print(_('Aserradero:\n'))
	elif aserraderoOk is False and bienOk is True:
		tipoDonacion = 2
		print('{}:\n'.format(bien))
	else:
		print(_('No se puede donar\n'))
		return

	tipo = tipo[tipoDonacion - 1]

	cantidad = read(min=0, max=int(madera), msg=_('Cantidad:'))
	s.post(payloadPost={'islandId': idIsla, 'type': tipo, 'action': 'IslandScreen', 'function': 'donate', 'donation': cantidad, 'backgroundView': 'island', 'templateView': 'resource', 'actionRequest': s.token(), 'ajax': '1'})

def printEstadoMina(s, url, bien):
	html = s.post(url)
	levels = re.search(r'"resourceLevel":"(\d+)","tradegoodLevel":"(\d+)"', html)
	if bien == _('Aserradero'):
		lv = levels.group(1)
	else:
		lv = levels.group(2)
	infoMina = re.search(r':<\\/h4>\\n\s*<ul\sclass=\\"resources\\">\\n\s*<li\sclass=\\"wood\\">([\d,]+)<[\s\S]*?:<\\/h4>\\n\s*<ul\sclass=\\"resources\\">\\n\s*<li\sclass=\\"wood\\">([\d,]+)<', html)
	if infoMina is not None:
		donado = infoMina.group(2)
		porDonar = infoMina.group(1)
		donado = int(donado.replace(',', ''))
		porDonar = int(porDonar.replace(',', ''))
		print('{} lv:{}'.format(bien, lv))
		print('{} / {} {}%'.format(addPuntos(donado), addPuntos(porDonar), addPuntos(int((100 * donado) / porDonar))))
	else:
		print(_('{}: Está ampliando al nivel {:d}\n').format(bien, int(lv) + 1))
	return infoMina is not None



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import gettext
import traceback
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.getJson import getCiudad
from ikabot.helpers.planearViajes import planearViajes
from ikabot.helpers.recursos import *
from ikabot.helpers.varios import addPuntos
from ikabot.helpers.process import forkear
from ikabot.helpers.gui import banner

t = gettext.translation('repartirRecurso',
                        localedir,
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def repartirRecurso(s):

	banner()

	print(_('¿Qué recurso quiere repartir?'))
	print(_('(1) Vino'))
	print(_('(2) Marmol'))
	print(_('(3) Cristal'))
	print(_('(4) Azufre'))
	recurso = read(min=1, max=4)

	recursoTotal = 0
	dict_idVino_diponible = {}
	(idsCiudades, ciudades) = getIdsDeCiudades(s)
	ciudadesOrigen = {}
	for idCiudad in idsCiudades:
		esTarget =  ciudades[idCiudad]['tradegood'] == str(recurso)
		if esTarget:
			html = s.get(urlCiudad + idCiudad)
			ciudad = getCiudad(html)
			recursos = getRecursosDisponibles(html)
			disponible = int(recursos[recurso]) - 1000 # dejo 1000 por las dudas
			ciudad['disponible'] = disponible if disponible > 0 else 0
			recursoTotal += ciudad['disponible']
			ciudadesOrigen[idCiudad] = ciudad
	aEnviar = len(ciudades) - len(ciudadesOrigen)
	recursoXciudad = int(recursoTotal / aEnviar)
	maximo = addPuntos(recursoXciudad)

	if recursoXciudad > 1000:
		maximo = maximo[:-3] + '000'

	print(_('Se puede enviar como máximo {} a cada ciudad').format(maximo))
	cantidad = read(msg=_('¿Cuanto enviar a cada ciudad?:'), min=0, max=recursoXciudad)

	if cantidad == 0:
		return

	print(_('\nPor enviar {} a cada ciudad').format(addPuntos(cantidad)))
	print(_('¿Proceder? [Y/n]'))
	rta = read(values=['y', 'Y', 'n', 'N', ''])
	if rta.lower() == 'n':
		return

	forkear(s)
	if s.padre is True:
		return

	rutas = []
	for idCiudadDestino in [ idCity for idCity in idsCiudades if idCity not in ciudadesOrigen ]:
		htmlD = s.get(urlCiudad + idCiudadDestino)
		ciudadD = getCiudad(htmlD)
		idIsla = ciudadD['islandId']
		faltante = cantidad
		for idCiudadOrigen in ciudadesOrigen:
			if faltante == 0:
				break
			ciudadO = ciudadesOrigen[idCiudadOrigen]
			recursoDisponible = ciudadO['disponible']
			for ruta in rutas:
				origen = ruta[0]
				rec = ruta[recurso + 3]
				if origen['id'] == idCiudadOrigen:
					recursoDisponible -= rec
			enviar = faltante if recursoDisponible > faltante else recursoDisponible
			ocupado   = getRecursosDisponibles(ciudadD['html'], num=True)[recurso]
			capacidad = getCapacidadDeAlmacenamiento(ciudadD['html'], num=True)
			disponible = capacidad - ocupado
			if disponible < enviar:
				faltante = 0
				enviar = disponible
			else:
				faltante -= enviar

			if recurso == 1:
				ruta = (ciudadO, ciudadD, idIsla, 0, enviar, 0, 0, 0)
			elif recurso == 2:
				ruta = (ciudadO, ciudadD, idIsla, 0, 0, enviar, 0, 0)
			elif recurso == 3:
				ruta = (ciudadO, ciudadD, idIsla, 0, 0, 0, enviar, 0)
			else:
				ruta = (ciudadO, ciudadD, idIsla, 0, 0, 0, 0, enviar)
			rutas.append(ruta)

	info = _('\nRepartir recurso\n')
	for ruta in rutas:
		(ciudadO, ciudadD, idIsla, md, vn, mr, cr, az) = ruta
		rec = ruta[recurso + 3]
		info = info + _('{} -> {}\n{}: {}\n').format(ciudadO['cityName'], ciudadD['cityName'], tipoDeBien[recurso], addPuntos(rec))
	setInfoSignal(s, info)
	try:
		planearViajes(s, rutas)
	except:
		msg = _('Error en:\n{}\nCausa:\n{}').format(info, traceback.format_exc())
		sendToBot(msg)
	finally:
		s.logout()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import gettext
import traceback
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.gui import *
from ikabot.helpers.process import forkear
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.getJson import getCiudad
from ikabot.helpers.recursos import getRecursosDisponibles

t = gettext.translation('botDonador', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def botDonador(s):
	banner()
	(idsCiudades, ciudades) = getIdsDeCiudades(s)
	ciudades_dict = {}
	bienes = {'1': '(V)', '2': '(M)', '3': '(C)', '4': '(A)'}
	for idCiudad in idsCiudades:
		html = s.get(urlCiudad + idCiudad)
		ciudad = getCiudad(html)
		tradegood = ciudades[idCiudad]['tradegood']
		bien = bienes[tradegood]
		print(_('En la ciudad {} {}, ¿Desea donar al aserradero o al bien de cambio? [a/b]').format(ciudad['cityName'], bien))
		rta = read(values=[_('a'), _('A'), _('b'), _('B')])
		tipo = 'resource' if rta.lower() == _('a') else 'tradegood'
		ciudades_dict[idCiudad] = {'isla': ciudad['islandId'], 'tipo': tipo}

	print(_('Se donará todos los días.'))
	enter()

	forkear(s)
	if s.padre is True:
		return

	info = _('\nDono todos los días\n')
	setInfoSignal(s, info)
	try:
		do_it(s, idsCiudades, ciudades_dict)
	except:
		msg = _('Error en:\n{}\nCausa:\n{}').format(info, traceback.format_exc())
		sendToBot(msg)
	finally:
		s.logout()

def do_it(s, idsCiudades, ciudades_dict):
	while True:
		for idCiudad in idsCiudades:
			html = s.get(urlCiudad + idCiudad)
			madera = getRecursosDisponibles(html)[0]
			idIsla = ciudades_dict[idCiudad]['isla']
			tipo = ciudades_dict[idCiudad]['tipo']
			s.post(payloadPost={'islandId': idIsla, 'type': tipo, 'action': 'IslandScreen', 'function': 'donate', 'donation': madera, 'backgroundView': 'island', 'templateView': 'resource', 'actionRequest': s.token(), 'ajax': '1'})
		msg = _('Doné automaticamente.')
		sendToBotDebug(msg, debugON_botDonador)
		time.sleep(24*60*60)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import os
import json
import random
import gettext
import ikabot.config as config
import ikabot.web.sesion
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.gui import enter
from ikabot.config import *

t = gettext.translation('botComm', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def sendToBotDebug(msg, debugON):
	if debugON:
		sendToBot(msg)

def sendToBot(msg, Token=False):
	if Token is False:
		msg = 'pid:{}\n{}\n{}'.format(os.getpid(), config.infoUser, msg)
	with open(config.telegramFile, 'r', os.O_NONBLOCK) as filehandler:
		text = filehandler.read()
	valid = re.search(r'\d{6,}:[A-Za-z0-9_-]{34,}\n\d{8,9}', text)
	if valid is not None:
		(botToken, chatId) = text.splitlines()
		ikabot.web.sesion.normal_get('https://api.telegram.org/bot{}/sendMessage'.format(botToken), params={'chat_id': chatId, 'text': msg})

def telegramFileValido():
	with open(config.telegramFile, 'r', os.O_NONBLOCK) as filehandler:
		text = filehandler.read()
	valid = re.search(r'\d{6,}:[A-Za-z0-9_-]{34,}\n\d{8,9}', text)
	return valid is not None

def getUserResponse():
	with open(config.telegramFile, 'r', os.O_NONBLOCK) as filehandler:
		text = filehandler.read()
	valid = re.search(r'\d{6,}:[A-Za-z0-9_-]{34,}\n\d{8,9}', text)
	if valid is not None:
		(botToken, chatId) = text.splitlines()
		updates = ikabot.web.sesion.normal_get('https://api.telegram.org/bot{}/getUpdates'.format(botToken)).text
		updates = json.loads(updates, strict=False)
		if updates['ok'] is False:
			return []
		updates = updates['result']
		return [update['message']['text'] for update in updates if update['message']['chat']['id'] == int(chatId)]

def botValido(s):
	if telegramFileValido():
		return True
	else:
		print(_('Debe proporcionar las credenciales válidas para comunicarse por telegram.'))
		print(_('Se requiere del token del bot a utilizar y de su chat_id'))
		print(_('Para más informacion sobre como obtenerlos vea al readme de https://github.com/physics-sp/ikabot'))
		rta = read(msg=_('¿Porporcionará las credenciales ahora? [y/N]'), values=['y','Y','n', 'N', ''])
		if rta.lower() != 'y':
			return False
		else:
			botToken = read(msg=_('Token del bot:'))
			chat_id = read(msg=_('Chat_id:'))
			with open(config.telegramFile, 'w', os.O_NONBLOCK) as filehandler:
				filehandler.write(botToken + '\n' + chat_id)
				filehandler.flush()
			rand = random.randint(1000, 9999)
			msg = _('El token a ingresar es:{:d}').format(rand)
			sendToBot(msg, Token=True)
			rta = read(msg=_('Se envio un mensaje por telegram, ¿lo recibió? [Y/n]'), values=['y','Y','n', 'N', ''])
			if rta.lower() == 'n':
				with open(config.telegramFile, 'w', os.O_NONBLOCK) as file:
					pass
				print(_('Revíse las credenciales y vuelva a proveerlas.'))
				enter()
				return False
			else:
				recibido = read(msg=_('Ingrese el token recibido mediante telegram:'), digit=True)
				if rand != recibido:
					with open(config.telegramFile, 'w', os.O_NONBLOCK) as file:
						pass
					print(_('El token es incorrecto'))
					enter()
					return False
				else:
					print(_('El token es correcto.'))
					enter()
					return True



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import gettext
from ikabot.config import *
from ikabot.web.getSesion import getSesion
from ikabot.helpers.gui import *
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.process import run
from ikabot.helpers.signals import setSignalsHandlers
from ikabot.funcion.subirEdificio import subirEdificios
from ikabot.funcion.menuRutaComercial import menuRutaComercial
from ikabot.funcion.repartirRecurso import repartirRecurso
from ikabot.funcion.getStatus import getStatus
from ikabot.funcion.donar import donar
from ikabot.funcion.buscarEspacios import buscarEspacios
from ikabot.funcion.entrarDiariamente import entrarDiariamente
from ikabot.funcion.alertarAtaques import alertarAtaques
from ikabot.funcion.botDonador import botDonador
from ikabot.funcion.update import update
from ikabot.funcion.alertarPocoVino import alertarPocoVino
from ikabot.funcion.comprarRecursos import comprarRecursos
from ikabot.funcion.modoVacaciones import modoVacaciones
from ikabot.funcion.activarMilagro import activarMilagro
from ikabot.funcion.entrenarTropas import entrenarTropas

t = gettext.translation('command_line', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def menu(s):
	banner()
	menu_actions = [
					subirEdificios,
					menuRutaComercial,
					repartirRecurso,
					getStatus,
					donar,
					buscarEspacios,
					entrarDiariamente,
					alertarAtaques,
					botDonador,
					alertarPocoVino,
					comprarRecursos,
					modoVacaciones,
					activarMilagro,
					entrenarTropas,
					update
					]
	print(_('(0)  Salir'))
	print(_('(1)  Lista de construcción'))
	print(_('(2)  Enviar recursos'))
	print(_('(3)  Repartir recurso'))
	print(_('(4)  Estado de la cuenta'))
	print(_('(5)  Donar'))
	print(_('(6)  Buscar espacios nuevos'))
	print(_('(7)  Entrar diariamente'))
	print(_('(8)  Alertar ataques'))
	print(_('(9)  Bot donador'))
	print(_('(10) Alertar poco vino'))
	print(_('(11) Comprar recursos'))
	print(_('(12) Activar modo vacaciones'))
	print(_('(13) Activar milagro'))
	print(_('(14) Entrenar tropas'))
	print(_('(15) Actualizar Ikabot'))
	entradas = len(menu_actions)
	eleccion = read(min=0, max=entradas)
	if eleccion != 0:
		try:
			menu_actions[eleccion - 1](s)
		except KeyboardInterrupt:
			pass
		menu(s)
	else:
		clear()

def inicializar():
	os.chdir(os.getenv("HOME"))
	if not os.path.isfile(cookieFile):
		open(cookieFile, 'w')
		os.chmod(cookieFile, 0o600)
	if not os.path.isfile(telegramFile):
		open(telegramFile, 'w')
		os.chmod(telegramFile, 0o600)

def start():
	inicializar()
	s = getSesion()
	setSignalsHandlers(s)
	try:
		menu(s)
	finally:
		if os.fork() == 0:
			s.logout()

def main():
	try:
		start()
	except KeyboardInterrupt:
		clear()

if __name__ == '__main__':
	main()



import os
import locale
import gettext

local = locale.setlocale(locale.LC_ALL, '')
if 'es_' in local:
	idiomas = ['es']
else:
	idiomas = ['en']

localedir = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'locale')
t = gettext.translation('config', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext


http_proxy  = "http://127.0.0.1:8080"
https_proxy = "https://127.0.0.1:8080"
proxyDict = {"http": http_proxy, "https": https_proxy}
proxyDict = {}
secure_traffic = True
ids = None
ciudades = None
menuCiudades = ''
infoUser = ''
cookieFile = '.cookies.ika'
telegramFile = '.telegram.ika'
urlCiudad = 'view=city&cityId='
urlIsla = 'view=island&islandId='
prompt = ' >>  '
tipoDeBien = [_('Madera'), _('Vino'), _('Marmol'), _('Cristal'), _('Azufre')]
ConnectionError_wait = 5 * 60
debugON_alertarAtaques    = False
debugON_alertarPocoVino   = False
debugON_botDonador        = False
debugON_buscarEspacios    = False
debugON_entrarDiariamente = False
debugON_enviarVino        = False
debugON_menuRutaComercial = False
debugON_subirEdificio     = False
debugON_session           = False
debugON_comprarRecursos   = False



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import gettext
import traceback
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.planearViajes import planearViajes
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.getJson import getCiudad
from ikabot.helpers.process import forkear
from ikabot.helpers.varios import addPuntos
from ikabot.helpers.recursos import *


def menuRutaComercial(s):
	t = gettext.translation('menuRutaComercial', 
	                        localedir, 
	                        languages=idiomas,
	                        fallback=True)
	_ = t.gettext
	rutas = []
	while True:

		banner()
		print(_('Ciudad de origen:'))
		try:
			ciudadO = elegirCiudad(s)
		except KeyboardInterrupt:
			if rutas:
				print(_('¿Enviar viajes? [Y/n]'))
				rta = read(values=['y', 'Y', 'n', 'N', ''])
				if rta.lower() != 'n':
					break
			return
		html = ciudadO['html']
		max = getRecursosDisponibles(html)
		total = list(map(int, max))

		banner()
		print(_('Ciudad de destino'))
		ciudadD = elegirCiudad(s, ajenas=True)
		idIsla = ciudadD['islandId']

		if ciudadO['id'] == ciudadD['id']:
			continue

		if ciudadD['propia']:
			html = ciudadD['html']
			mad, vin, mar, cri, azu = getRecursosDisponibles(html, num=True)
			capacidad = getCapacidadDeAlmacenamiento(html)
			capacidad = int(capacidad)
			mad = capacidad - mad
			vin = capacidad - vin
			mar = capacidad - mar
			cri = capacidad - cri
			azu = capacidad - azu

		resto = total
		for ruta in rutas:
			(origen, destino, __, md, vn, mr, cr, az) = ruta
			if origen['id'] == ciudadO['id']:
				resto = (resto[0] - md, resto[1] - vn, resto[2] - mr, resto[3] - cr, resto[4] - az)
			if ciudadD['propia'] and destino['id'] == ciudadD['id']:
				mad = mad - md
				vin = vin - vn
				mar = mar - mr
				cri = cri - cr
				azu = azu - az

		banner()
		if ciudadD['propia']:
			msg = ''
			if resto[0] > mad:
				msg += _('{} más de madera\n').format(addPuntos(mad))
			if resto[1] > vin:
				msg += _('{} más de vino\n').format(addPuntos(vin))
			if resto[2] > mar:
				msg += _('{} más de marmol\n').format(addPuntos(mar))
			if resto[3] > cri:
				msg += _('{} más de cristal\n').format(addPuntos(cri))
			if resto[4] > azu:
				msg += _('{} más de azufre\n').format(addPuntos(azu))
			if msg:
				print(_('Solo puede almacenar:\n{}').format(msg))
		print(_('Disponible:'))
		print(_('Madera {} Vino {} Marmol {} Cristal {} Azufre {}').format(addPuntos(resto[0]), addPuntos(resto[1]), addPuntos(resto[2]), addPuntos(resto[3]), addPuntos(resto[4])))
		print(_('Enviar:'))
		try:
			md = pedirValor(_(' Madera:'), resto[0])
			vn = pedirValor(_('   Vino:'), resto[1])
			mr = pedirValor(_(' Marmol:'), resto[2])
			cr = pedirValor(_('Cristal:'), resto[3])
			az = pedirValor(_(' Azufre:'), resto[4])
		except KeyboardInterrupt:
			continue
		if md + vn + mr + cr + az == 0:
			continue

		banner()
		print(_('Por enviar de {} a {}').format(ciudadO['cityName'], ciudadD['cityName']))
		enviado = ''
		if md:
			enviado += _('Madera:{} ').format(addPuntos(md))
		if vn:
			enviado += _('Vino:{} ').format(addPuntos(vn))
		if mr:
			enviado += _('Marmol:{} ').format(addPuntos(mr))
		if cr:
			enviado += _('Cristal:{} ').format(addPuntos(cr))
		if az:
			enviado += _('Azufre:{}').format(addPuntos(az))
		print(enviado)
		print(_('¿Proceder? [Y/n]'))
		rta = read(values=['y', 'Y', 'n', 'N', ''])
		if rta.lower() != 'n':
			ruta = (ciudadO, ciudadD, idIsla, md, vn, mr, cr, az)
			rutas.append(ruta)
			print(_('¿Realizar otro envio? [y/N]'))
			rta = read(values=['y', 'Y', 'n', 'N', ''])
			if rta.lower() != 'y':
				break

	forkear(s)
	if s.padre is True:
		return

	info = _('\nRuta comercial\n')
	for ruta in rutas:
		(ciudadO, ciudadD, idIsla, md, vn, mr, cr, az) = ruta
		info = info + _('{} -> {}\nMadera: {} Vino: {} Marmol: {} Cristal: {} Azufre: {}\n').format(ciudadO['cityName'], ciudadD['cityName'], addPuntos(md), addPuntos(vn), addPuntos(mr), addPuntos(cr), addPuntos(az))

	setInfoSignal(s, info)
	try:
		msg  = _('Comienzo a enviar recursos:\n')
		msg += info
		sendToBotDebug(msg, debugON_menuRutaComercial)

		planearViajes(s, rutas)

		msg  = _('Termino de enviar recursos:\n')
		msg += info
		sendToBotDebug(msg, debugON_menuRutaComercial)
	except:
		msg = _('Error en:\n{}\nCausa:\n{}').format(info, traceback.format_exc())
		sendToBot(msg)
	finally:
		s.logout()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import sys
import json
import gettext
from decimal import *
from ikabot.config import *
from ikabot.helpers.getJson import *
from ikabot.helpers.gui import *

t = gettext.translation('pedirInfo', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

getcontext().prec = 30

def read(min=None, max=None, digit=False, msg=prompt, values=None, empty=False): # lee input del usuario
	def _invalido():
		print('\033[1A\033[K', end="") # Borro linea
		return read(min, max, digit, msg, values)

	try:
		leido = input(msg)
	except EOFError:
		return _invalido()

	if leido == '' and empty is True:
		return leido

	if digit is True or min is not None or max is not None:
		if leido.isdigit() is False:
			return _invalido()
		else:
			try:
				leido = eval(leido)
			except SyntaxError:
				return _invalido()
	if min is not None and leido < min:
		return _invalido()
	if max is not None and leido > max:
		return _invalido()
	if values is not None and leido not in values:
		return _invalido()
	return leido

def elegirCiudad(s, ajenas=False):
	global menuCiudades
	(ids, ciudades) = getIdsDeCiudades(s)
	if menuCiudades == '':
		maxNombre = 0
		for unId in ids:
			largo = len(ciudades[unId]['name'])
			if largo > maxNombre:
				maxNombre = largo
		pad = lambda name: ' ' * (maxNombre - len(name) + 2)
		bienes = {'1': '(V)', '2': '(M)', '3': '(C)', '4': '(A)'}
		prints = []
		i = 0
		if ajenas:
			print(_(' 0: ciudad ajena'))
		else:
			print('')
		for unId in ids:
			i += 1
			tradegood = ciudades[unId]['tradegood']
			bien = bienes[tradegood]
			nombre = ciudades[unId]['name']
			num = ' ' + str(i) if i < 10 else str(i)
			menuCiudades += '{}: {}{}{}\n'.format(num, nombre, pad(nombre), bien)
		menuCiudades = menuCiudades[:-1]
	if ajenas:
		print(_(' 0: ciudad ajena'))
	print(menuCiudades)

	if ajenas:
		eleccion = read(min=0, max=len(ids))
	else:
		eleccion = read(min=1, max=len(ids))
	if eleccion == 0:
		return elegirCiudadAjena(s)
	else:
		html = s.get(urlCiudad + ids[eleccion -1])
		return getCiudad(html)

def elegirCiudadAjena(s):
	banner()
	x = read(msg='coordenada x:', digit=True)
	y = read(msg='coordenada y:', digit=True)
	print('')
	url = 'view=worldmap_iso&islandX={}&islandY={}&oldBackgroundView=island&islandWorldviewScale=1'.format(x, y)
	html = s.get(url)
	try:
		jsonIslas = re.search(r'jsonData = \'(.*?)\';', html).group(1)
		jsonIslas = json.loads(jsonIslas, strict=False)
		idIsla = jsonIslas['data'][str(x)][str(y)][0]
	except:
		print(_('Coordenadas incorrectas'))
		enter()
		banner()
		return elegirCiudad(s, ajenas=True)
	html = s.get(urlIsla + idIsla)
	isla = getIsla(html)
	maxNombre = 0
	for ciudad in isla['cities']:
		if ciudad['type'] == 'city':
			largo = len(ciudad['name'])
			if largo > maxNombre:
				maxNombre = largo
	pad = lambda name: ' ' * (maxNombre - len(name) + 2)
	i = 0
	opciones = []
	for ciudad in isla['cities']:
		if ciudad['type'] == 'city' and ciudad['state'] == '' and ciudad['Name'] != s.username:
			i += 1
			num = ' ' + str(i) if i < 10 else str(i)
			print('{}: {}{}({})'.format(num, ciudad['name'], pad(ciudad['name']), ciudad['Name']))
			opciones.append(ciudad)
	if i == 0:
		print(_('No hay ciudades donde enviar recursos en esta isla'))
		enter()
		return elegirCiudad(s, ajenas=True)
	eleccion = read(min=1, max=i)
	ciudad = opciones[eleccion - 1]
	ciudad['islandId'] = isla['id']
	ciudad['cityName'] = ciudad['name']
	ciudad['propia'] = False
	return ciudad

def getEdificios(s, idCiudad):
	html = s.get(urlCiudad + idCiudad)
	ciudad = getCiudad(html)
	i = 0
	pos = -1
	prints = []
	posiciones = []
	prints.append(_('(0)\t\tsalir'))
	posiciones.append(None)
	for posicion in ciudad['position']:
		pos += 1
		if posicion['name'] != 'empty':
			i += 1
			level = posicion['level']
			if int(level) < 10:
				level = ' ' + level
			if posicion['isBusy']:
				level = level + '+'
			prints.append(_('({:d})\tlv:{}\t{}').format(i, level, posicion['name']))
			posiciones.append(pos)
	eleccion = menuEdificios(prints, ciudad, posiciones)
	return eleccion

def menuEdificios(prints, ciudad, posiciones):
	banner()
	for textoEdificio in prints:
		print(textoEdificio)

	eleccion = read(min=0, max=len(prints)-1)

	if eleccion == 0:
		return []
	posicion = posiciones[eleccion]
	nivelActual = int(ciudad['position'][posicion]['level'])
	if ciudad['position'][posicion]['isBusy']:
		nivelActual += 1

	banner()
	print(_('edificio:{}').format(ciudad['position'][posicion]['name']))
	print(_('nivel actual:{}').format(nivelActual))

	nivelFinal = read(min=nivelActual, msg=_('subir al nivel:'))

	niveles = nivelFinal - nivelActual
	rta = []
	for i in range(0, niveles):
		rta.append(posicion)
	return rta

def pedirValor(text, max):
	var = read(msg=text, min=0, max=max, empty=True)
	if var == '':
		var = 0
	return var

def getIdsDeCiudades(s, all=False):
	global ciudades
	global ids
	if ids is None or ciudades is None or s.padre is False:
		html = s.get()
		ciudades = re.search(r'relatedCityData:\sJSON\.parse\(\'(.+?),\\"additionalInfo', html).group(1) + '}'
		ciudades = ciudades.replace('\\', '')
		ciudades = ciudades.replace('city_', '')
		ciudades = json.loads(ciudades, strict=False)

		ids = [ciudad for ciudad in ciudades]
		ids = sorted(ids)

	# {'coords': '[x:y] ', 'id': idCiudad, 'tradegood': '..', 'name': 'nomberCiudad', 'relationship': 'ownCity'|'occupiedCities'|..}
	if all is False:
		ids_own   = [ciudad for ciudad in ciudades if ciudades[ciudad]['relationship'] == 'ownCity']
		ids_other = [ciudad for ciudad in ciudades if ciudades[ciudad]['relationship'] != 'ownCity']
		ciudades_own = ciudades.copy()
		for id in ids_other:
			del ciudades_own[id]
		return (ids_own, ciudades_own)
	else:
		return (ids, ciudades)

def getIdsdeIslas(s):
	(idsCiudades, ciudades) = getIdsDeCiudades(s)
	idsIslas = set()
	for idCiudad in idsCiudades:
		html = s.get(urlCiudad + idCiudad)
		ciudad = getCiudad(html)
		idIsla = ciudad['islandId']
		idsIslas.add(idIsla)
	return list(idsIslas)



import setuptools

with open("README.md", "r") as fh:
	long_description = fh.read()

setuptools.setup(
	name="ikabot",
	version="4.7.0",
	author="physics-sp",
	description="A bot for ikariam",
	long_description=long_description,
	long_description_content_type="text/markdown",
	url="https://github.com/physics-sp/ikabot",
	include_package_data=True,
	packages=setuptools.find_packages(),
	install_requires=[
		  'requests',
		  'pycryptodome'
	],
	entry_points = {
		'console_scripts': ['ikabot=ikabot.command_line:main'],
	},
	classifiers=(
		"Programming Language :: Python :: 3",
		"License :: OSI Approved :: MIT License",
		"Operating System :: POSIX :: Linux",
	),
)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import time
import traceback
import gettext
from decimal import *
from ikabot.config import *
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.process import forkear
from ikabot.helpers.gui import enter
from ikabot.helpers.pedirInfo import getIdsDeCiudades
from ikabot.helpers.varios import diasHorasMinutos
from ikabot.helpers.recursos import *
from ikabot.helpers.botComm import *


t = gettext.translation('alertarPocoVino',
                        localedir,
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

getcontext().prec = 30

def alertarPocoVino(s):
	if botValido(s) is False:
		return
	horas = read(msg=_('¿Cuántas horas deben quedar hasta que se acabe el vino en una ciudad para que es dé aviso?: '),min=1)
	print(_('Se avisará cuando el vino se acabe en {:d} horas en alguna ciudad.').format(horas))
	enter()

	forkear(s)
	if s.padre is True:
		return

	info = _('\nAviso si el vino se acaba en {:d} horas\n').format(horas)
	setInfoSignal(s, info)
	try:
		do_it(s, horas)
	except:
		msg = _('Error en:\n{}\nCausa:\n{}').format(info, traceback.format_exc())
		sendToBot(msg)
	finally:
		s.logout()

def do_it(s, horas):
	while True:

		ids, ciudades = getIdsDeCiudades(s)
		for city in ciudades:
			if 'avisado' not in ciudades[city]:
				ciudades[city]['avisado'] = False

		for city in ciudades:
			if ciudades[city]['tradegood'] == '1':
				continue

			id = str(ciudades[city]['id'])
			html = s.get(urlCiudad + id)
			consumoXhr = getConsumoDeVino(html)
			consumoXseg = Decimal(consumoXhr) / Decimal(3600)
			max = getRecursosDisponibles(html)
			if consumoXseg == 0:
				if ciudades[city]['avisado'] is False:
					msg = _('La ciudad {} no esta consumiendo vino!').format(ciudades[city]['name'])
					sendToBot(msg)
					ciudades[city]['avisado'] = True
				continue
			segsRestantes = Decimal(int(max[1])) / Decimal(consumoXseg)

			if segsRestantes < horas*60*60:
				if ciudades[city]['avisado'] is False:
					tiempoRestante = diasHorasMinutos(segsRestantes)
					msg = _('En {} se acabará el vino en {}').format(tiempoRestante, ciudades[city]['name'])
					sendToBot(msg)
					ciudades[city]['avisado'] = True
			else:
				ciudades[city]['avisado'] = False
		time.sleep(20*60)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import time
from decimal import *

getcontext().prec = 30

def addPuntos(num):
	return '{0:,}'.format(int(num)).replace(',','.')

def diasHorasMinutos(segundosTotales):
	dias = int(segundosTotales / Decimal(86400))
	segundosTotales -= dias * Decimal(86400)
	horas = int(segundosTotales / Decimal(3600))
	segundosTotales -= horas * Decimal(3600)
	minutos = int(segundosTotales / Decimal(60))
	texto = ''
	if dias > 0:
		texto = str(dias) + 'D '
	if horas > 0:
		texto = texto + str(horas) + 'H '
	if minutos > 0 and dias == 0:
		texto = texto + str(minutos) + 'M '
	return texto

def esperar(segundos):
	ratio = (1 + 5 ** 0.5) / 2 - 1
	comienzo = time.time()
	fin = comienzo + segundos
	restantes = segundos
	while restantes > 0:
		time.sleep(restantes * ratio)
		restantes = fin - time.time()
	return
